#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>

#include <Looper.h>
#include <Node.h>
#include <NodeMonitor.h>


struct Looper : BLooper {
	Looper(const char* path)
		:
		BLooper(),
		fPath(path)
	{
	}

	virtual void MessageReceived(BMessage* message)
	{
		if (message->what != B_NODE_MONITOR)
			return BLooper::MessageReceived(message);

		if (message->GetInt32("opcode", 0) == B_STAT_CHANGED
			&& (message->GetInt32("fields", 0) & B_STAT_MODIFICATION_TIME)
				!= 0) {
			struct stat st;
			if (lstat(fPath, &st) == 0) {
				printf("node %" B_PRIdDEV ":%" B_PRIdINO " modification time "
					"changed: %ld.%ld\n", st.st_dev, st.st_ino,
					(long)st.st_mtim.tv_sec, (long)st.st_mtim.tv_nsec);
			}
		}
	}

private:
	const char* fPath;
};


int
main(int argc, const char* const* argv)
{
	if (argc < 2) {
		fprintf(stderr, "Usage: %s <path>\n", argv[0]);
		return 1;
	}

	const char* path = argv[1];

	struct stat st;
	if (lstat(path, &st) != 0) {
		fprintf(stderr, "Error: Failed to stat() \"%s\": %s\n", path,
			strerror(errno));
		return 1;
	}

	Looper* looper = new Looper(path);
	thread_id thread = looper->Run();

	node_ref nodeRef(st.st_dev, st.st_ino);
	status_t error = watch_node(&nodeRef, B_WATCH_STAT, looper);
	if (error != B_OK) {
		fprintf(stderr, "Error: Failed to start watching node: %s\n",
			strerror(error));
		return 1;
	}

	wait_for_thread(thread, NULL);

	return 0;
}
